"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const subscription_1 = require("./subscription");
const interceptor_1 = require("./interceptor");
/**
 * @class ModuleReference
 * @author Jonathan Casarrubias
 * @license MIT
 */
class MethodReference {
    /**
     * @constructor
     * @param name
     * @param componentReference
     */
    constructor(name, componentReference) {
        this.name = name;
        this.componentReference = componentReference;
    }
    /**
     * @method call
     * @param payload
     * @description This method will call for RPC endpoints. It will send an application operation
     * to the OnixJS Service HOST.
     */
    async call(payload, filter) {
        return new Promise(async (resolve, reject) => {
            if (this.invalid('rpc')) {
                reject(new Error(`ONIXJS CLIENT: Unable to call ${this.endpoint()}, RPC doesn't exist on OnixJS Server`));
            }
            else {
                let operation = {
                    uuid: utils_1.Utils.uuid(),
                    type: 13 /* ONIX_REMOTE_CALL_PROCEDURE */,
                    message: {
                        rpc: this.endpoint(),
                        request: {
                            metadata: {
                                filter,
                                stream: false,
                                caller: this.componentReference.moduleReference.appReference
                                    .config.claims.sub,
                                token: this.componentReference.moduleReference.appReference
                                    .config.token,
                                register: this.componentReference.moduleReference.appReference.config.registration()
                                    .uuid,
                            },
                            payload,
                        },
                    },
                };
                const listenerId = this.componentReference.moduleReference.appReference.config.listeners
                    .namespace('remote')
                    .add(async (response) => {
                    if (response.uuid === operation.uuid &&
                        response.type ===
                            14 /* ONIX_REMOTE_CALL_PROCEDURE_RESPONSE */) {
                        this.componentReference.moduleReference.appReference.config.listeners.remove(listenerId);
                        // Call for after interceptor
                        if (interceptor_1.Interceptors.after) {
                            response = await interceptor_1.Interceptors.after(response);
                        }
                        resolve(response.message.request.payload);
                    }
                });
                // Call for before interceptor
                if (interceptor_1.Interceptors.before) {
                    operation = await interceptor_1.Interceptors.before(operation);
                }
                // Send Operation to Server
                this.componentReference.moduleReference.appReference.config.client.send(JSON.stringify(operation));
            }
        });
    }
    /**
     * @method stream
     * @param listener
     * @param filter
     * @description This method will register a stream, which will be populated as the server keeps
     * sending chunks of information.
     */
    async stream(listener, filter) {
        return new Promise(resolve => {
            if (this.invalid('stream')) {
                listener(new Error(`ONIXJS CLIENT: Unable to call ${this.endpoint()}, RPC doesn't exist on OnixJS Server`));
            }
            else {
                const operation = {
                    uuid: utils_1.Utils.uuid(),
                    type: 13 /* ONIX_REMOTE_CALL_PROCEDURE */,
                    message: {
                        rpc: this.endpoint(),
                        request: {
                            metadata: {
                                filter,
                                stream: true,
                                caller: this.componentReference.moduleReference.appReference
                                    .config.claims.sub,
                                token: this.componentReference.moduleReference.appReference
                                    .config.token,
                                register: this.componentReference.moduleReference.appReference.config.registration()
                                    .uuid,
                            },
                            payload: undefined,
                        },
                    },
                };
                // Register Stream
                this.componentReference.moduleReference.appReference.config.client.send(JSON.stringify(operation));
                // Chunks of information will be received in a future
                const id = this.componentReference.moduleReference.appReference.config.listeners
                    .namespace('remote')
                    .add((response) => {
                    if (response.uuid === operation.uuid) {
                        if (response.type === 12 /* ONIX_REMOTE_CALL_STREAM */) {
                            listener(response.message.request.payload);
                        }
                        else if (response.type ===
                            15 /* ONIX_REMOTE_CALL_STREAM_SUBSCRIBED */) {
                            resolve(new subscription_1.Subscription(id, response.message.request.metadata.listener, this.endpoint(), operation, this.componentReference.moduleReference.appReference.config));
                        }
                    }
                });
            }
        });
    }
    invalid(type) {
        return (!this.componentReference.moduleReference.appReference.config.modules[this.componentReference.moduleReference.name] ||
            !this.componentReference.moduleReference.appReference.config.modules[this.componentReference.moduleReference.name] ||
            !this.componentReference.moduleReference.appReference.config.modules[this.componentReference.moduleReference.name][this.componentReference.name] ||
            this.componentReference.moduleReference.appReference.config.modules[this.componentReference.moduleReference.name][this.componentReference.name][this.name] !== type);
    }
    endpoint() {
        return `${this.componentReference.moduleReference.appReference.config.name}.${this.componentReference.moduleReference.name}.${this.componentReference.name}.${this.name}`;
    }
}
exports.MethodReference = MethodReference;
//# sourceMappingURL=method.reference.js.map