import { OnixClientConfig, IClaims } from './interfaces';
import { AppReference } from './core/app.reference';
import { ClientRegistration } from './core/client.registration';
export * from './core';
export * from './enums';
export * from './interfaces';
export declare const namespace = "onixjs:sdk";
/**
 * @class OnixClient
 * @author Jonathan Casarrubias <gh: mean-expert-official>
 * @license MIT
 * @description This class provides core functionality for
 * client applications.
 */
export declare class OnixClient {
    private config;
    private ws;
    private http;
    private storage;
    private listeners;
    private schema;
    private references;
    protected registration: ClientRegistration;
    /**
     * @constructor
     * @param config
     * @description Receives a client configuration, it will
     * also define a default value for our config incase is not
     * provided.
     */
    constructor(config: OnixClientConfig);
    /**
     * @method init
     * @description this method will get the onix infrastructure schema
     * in order to correctly configure each Application Reference.
     */
    init(): Promise<boolean>;
    /**
     * @method onDisconnect
     * @param handler
     * @description This method must be used to handle disconnections.
     * Any reference must be destroyed and re-created when the client
     * is connected again.
     */
    onDisconnect(handler: any): void;
    /**
     * @method reconnect
     * @param handler
     * @description This method must be used to handle re-connections.
     * Any reference must be destroyed and re-created when the client
     * is connected again.
     */
    reconnect(handler: any): void;
    /**
     * @method connect
     * @description This method will be internally used to handle ws connection
     * on init or under unexpected disconnections.
     */
    private connect();
    /**
     * @method disconnected
     * @param e
     * @description This method will notify disconnections. When notifying
     * the subscribers, they must dispose subscriptions and run the sdk
     * initialization again.
     */
    private disconnected(e);
    /**
     * @method onerror
     * @param e
     * @description This method will log errored disconnections
     */
    private onerror(e);
    /**
     * @method onclose
     * @param e
     * @description This method will log close disconnections
     */
    private onclose(e);
    /**
     * @method
     * @param resolve
     */
    private register(resolve, reject);
    /**
     * @method ping
     */
    private ping(id);
    /**
     * @method disconnect
     * @description Disconnect from websocket server
     */
    disconnect(): void;
    /**
     * @class AppReference
     * @param name
     * @description This method will construct an application reference.
     * Only if the provided schema defines it does exist.
     */
    AppReference(name: string): Promise<AppReference>;
    /**
     * @description This getter will return a stored access token
     * from the local storage adapter.
     */
    /**
     * @description This setter will store a provided access token
     * into the local storage adapter.
     */
    token: string;
    /**
     * @method claims
     * @param organization
     * @author Jonathan Casarrubias
     * @description This method will return an OIDC claims object.
     * Usually will provide the user information and any scope
     * defined within the OIDC Client.
     *
     * Organization param is optional and is created to allow devs
     * to claim user organization groups -if scoped-
     */
    claims(organization?: string): Promise<IClaims>;
    /**
     * @method setClaims
     * @description This method will directly set the claims from a user
     * without calling the SSO.
     *
     * This is an advanced method and must be avoided in most of the cases,
     * though it is public, should only be used in edge cases.
     */
    setClaims(claims: any): void;
    waitForConnection(callback: any, interval?: number): void;
    /**
     * @method logout
     * @description this method will clear the local storage, therefore
     * cleaning any stored information like token or claims.
     */
    logout(): void;
}
