"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const app_reference_1 = require("./core/app.reference");
const utils_1 = require("./utils");
const listener_collection_1 = require("./core/listener.collection");
const client_registration_1 = require("./core/client.registration");
__export(require("./core"));
__export(require("./enums"));
__export(require("./interfaces"));
exports.namespace = 'onixjs:sdk';
/**
 * @class OnixClient
 * @author Jonathan Casarrubias <gh: mean-expert-official>
 * @license MIT
 * @description This class provides core functionality for
 * client applications.
 */
class OnixClient {
    /**
     * @constructor
     * @param config
     * @description Receives a client configuration, it will
     * also define a default value for our config incase is not
     * provided.
     */
    constructor(config) {
        this.config = config;
        this.listeners = new listener_collection_1.ListenerCollection();
        this.schema = {}; // TODO Interface Schema
        this.references = {}; // Todo Reference Interface
        if (this.config.adapters.http &&
            this.config.adapters.websocket &&
            this.config.adapters.storage) {
            this.http = new this.config.adapters.http();
            this.ws = new this.config.adapters.websocket();
            this.storage = new this.config.adapters.storage();
            // Set some default configs
            if (!this.config.prefix) {
                this.config.prefix = exports.namespace;
            }
            if (!this.config.intervals) {
                this.config.intervals = {};
            }
            if (!this.config.intervals.ping) {
                this.config.intervals.ping = 5000;
            }
            if (!this.config.intervals.timeout) {
                this.config.intervals.timeout = 1000;
            }
            if (!this.config.intervals.reconnect) {
                this.config.intervals.reconnect = 5000;
            }
            if (!this.config.attempts) {
                this.config.attempts = {};
            }
            if (!this.config.attempts.ping) {
                this.config.attempts.ping = 3;
            }
        }
        else {
            console.log('ONIXJS SDK: Unable to find suitable adapters.');
        }
    }
    /**
     * @method init
     * @description this method will get the onix infrastructure schema
     * in order to correctly configure each Application Reference.
     */
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // Get OnixJS Schema
                this.schema = yield this.http.get(`${this.config.host}:${this.config.port}/.well-known/onixjs-schema`);
                // Connect
                return this.connect();
            }
            catch (e) {
                const error = new Error(`${exports.namespace} Unable to get host schema, verify internet connection and/or sdk host:port configs.`);
                this.disconnected(error);
                throw error;
            }
        });
    }
    /**
     * @method onDisconnect
     * @param handler
     * @description This method must be used to handle disconnections.
     * Any reference must be destroyed and re-created when the client
     * is connected again.
     */
    onDisconnect(handler) {
        this.listeners.namespace('disconnect').add(e => {
            handler(4);
        });
    }
    /**
     * @method reconnect
     * @param handler
     * @description This method must be used to handle re-connections.
     * Any reference must be destroyed and re-created when the client
     * is connected again.
     */
    reconnect(handler) {
        this.listeners.namespace('reconnect').add(handler);
    }
    /**
     * @method connect
     * @description This method will be internally used to handle ws connection
     * on init or under unexpected disconnections.
     */
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                // URL
                const url = `${this.config.port === 443 ? 'wss' : 'ws'}://${this.config.host.replace(/http[s]{0,1}:\/\//, '')}:${this.config.port}`;
                // Connect WebSocket
                this.ws.connect(url);
                // Register Single WS Listener
                this.ws.on('message', (data) => this.listeners
                    .namespace('remote')
                    .broadcast(utils_1.Utils.IsJsonString(data) ? JSON.parse(data) : data));
                // When connection is open then register and resolve
                this.ws.open(() => 
                // Wait until the connection is valid
                this.waitForConnection(() => {
                    // Set a ping interval process
                    const id = setInterval(() => this.ping(id), this.config.intervals.ping);
                    // Register current connection
                    this.register(resolve, reject);
                }));
                // Handle error disconnections
                this.ws.on('close', (e) => {
                    switch (e.code) {
                        case 1000: // CLOSE_NORMAL
                            console.log('WebSocket: closed');
                            break;
                        default:
                            // Abnormal closure
                            this.disconnected(e);
                            this.onclose(e);
                            break;
                    }
                });
                // Handle close disconnections
                this.ws.on('error', (e) => {
                    switch (e.code) {
                        case 'ECONNREFUSED':
                            this.disconnected(e);
                            break;
                        default:
                            this.onerror(e);
                            break;
                    }
                });
            });
        });
    }
    /**
     * @method disconnected
     * @param e
     * @description This method will notify disconnections. When notifying
     * the subscribers, they must dispose subscriptions and run the sdk
     * initialization again.
     */
    disconnected(e) {
        // Remove any websocket listener
        if (this.ws && this.ws.client && this.ws.removeAllListeners)
            this.ws.removeAllListeners();
        // Wait some time before notifying the
        // disconnection.
        // Will notify the disconnect listeners
        // the client implementing the SDK should
        // notify users there is a disconnection
        this.listeners.namespace('disconnect').broadcast(e);
        this.listeners.removeNameSpaceListeners('remote');
        this.references = {};
        setTimeout(() => {
            // Will notify the reconnect listeners
            // the client implementing the SDK should
            // Call the init method again
            this.listeners.namespace('reconnect').broadcast(e);
        }, this.config.intervals.reconnect);
    }
    /**
     * @method onerror
     * @param e
     * @description This method will log errored disconnections
     */
    onerror(e) {
        console.log('WebSocketClient: error', arguments);
    }
    /**
     * @method onclose
     * @param e
     * @description This method will log close disconnections
     */
    onclose(e) {
        console.log('WebSocketClient: closed', arguments);
    }
    /**
     * @method
     * @param resolve
     */
    register(resolve, reject) {
        const uuid = utils_1.Utils.uuid();
        // Register Client
        const operation = {
            uuid,
            type: 18 /* ONIX_REMOTE_REGISTER_CLIENT */,
            message: {
                rpc: 'register',
                request: {
                    metadata: {
                        stream: false,
                        register: uuid,
                    },
                    payload: {},
                },
            },
        };
        // Create listener
        const index = this.listeners
            .namespace('remote')
            .add((data) => {
            // Verify we actually get an object
            const response = (typeof data ===
                'string' && utils_1.Utils.IsJsonString(data)
                ? JSON.parse(data)
                : data);
            // Verify we got the result, which will provide the registration
            // Later might be used on handled disconnections.
            if (response.uuid === operation.uuid &&
                response.type === 19 /* ONIX_REMOTE_REGISTER_CLIENT_RESPONSE */) {
                if (response.message.request.payload.code &&
                    response.message.request.payload.message) {
                    reject(response.message.request.payload);
                }
                else {
                    this.registration = new client_registration_1.ClientRegistration(uuid);
                    this.listeners.remove(index);
                    resolve();
                }
            }
        });
        // Send registration operation
        this.ws.send(JSON.stringify(operation));
    }
    /**
     * @method ping
     */
    ping(id) {
        const ts = Math.round(new Date().getTime() / 1000).toString();
        let to = 0;
        // set timeout interval
        const interval = setInterval(() => {
            if (to >= this.config.attempts.ping) {
                this.disconnect();
                clearInterval(id);
                clearInterval(interval);
            }
            to += 1;
        }, this.config.intervals.timeout);
        const index = this.listeners
            .namespace('remote')
            .add((data) => {
            if ((typeof data === 'number' && data.toString().includes(ts)) ||
                (typeof data === 'string' && data.includes(ts))) {
                to = 0;
                this.listeners.namespace('remote').remove(index);
                clearInterval(interval);
            }
        });
        // Send timestamp
        this.ws.send(ts);
    }
    /**
     * @method disconnect
     * @description Disconnect from websocket server
     */
    disconnect() {
        this.disconnected('CLOSED');
        this.ws.close();
    }
    /**
     * @class AppReference
     * @param name
     * @description This method will construct an application reference.
     * Only if the provided schema defines it does exist.
     */
    AppReference(name) {
        return __awaiter(this, void 0, void 0, function* () {
            // Verify that the application actually exists on server
            if (!this.schema[name]) {
                throw new Error(`${exports.namespace} Application with ${name} doesn't exist on the OnixJS Server Environment.`);
            }
            // If the reference still doesn't exist, then create one
            if (!this.references[name]) {
                // Use passed host config if any
                this.references[name] = new app_reference_1.AppReference(Object.assign({
                    name,
                    client: this.ws,
                    token: this.token,
                    claims: yield this.claims(),
                    listeners: this.listeners,
                    registration: () => this.registration,
                }, this.schema[name]));
            }
            // Otherwise return a singleton instance of the reference
            return this.references[name];
        });
    }
    /**
     * @description This setter will store a provided access token
     * into the local storage adapter.
     */
    set token(token) {
        this.storage.setItem(`${this.config.prefix}:access_token`, token);
    }
    /**
     * @description This getter will return a stored access token
     * from the local storage adapter.
     */
    get token() {
        return this.storage.getItem(`${this.config.prefix}:access_token`) || '';
    }
    /**
     * @method claims
     * @param organization
     * @author Jonathan Casarrubias
     * @description This method will return an OIDC claims object.
     * Usually will provide the user information and any scope
     * defined within the OIDC Client.
     *
     * Organization param is optional and is created to allow devs
     * to claim user organization groups -if scoped-
     */
    claims(organization) {
        return __awaiter(this, void 0, void 0, function* () {
            // Load claims from local storage
            const persisted = this.storage.getItem(`${this.config.prefix}:claims`);
            // Verify that we already have an actual claims
            if (persisted) {
                return JSON.parse(persisted);
            }
            // Otherwise verify we actually have an access_token
            if (this.token.length > 0) {
                // Now call from the SSO the user claims
                const claims = yield this.http.get(`https://sso.onixjs.io/me?access_token=${this.token}${organization ? `&organization=${organization}` : ''}`);
                // Store now in localstorage
                this.setClaims(claims);
                // Return the claims
                return claims;
            }
            else {
                // This guy is not even logged, return an anonymous claim
                return { sub: '$anonymous' };
            }
        });
    }
    /**
     * @method setClaims
     * @description This method will directly set the claims from a user
     * without calling the SSO.
     *
     * This is an advanced method and must be avoided in most of the cases,
     * though it is public, should only be used in edge cases.
     */
    setClaims(claims) {
        // Store now in localstorage
        this.storage.setItem(`${this.config.prefix}:claims`, JSON.stringify(claims));
    }
    waitForConnection(callback, interval = 100) {
        if (this.ws.client.readyState === 1) {
            callback();
        }
        else {
            // optional: implement backoff for interval here
            setTimeout(() => {
                this.waitForConnection(callback, interval);
            }, interval);
        }
    }
    /**
     * @method logout
     * @description this method will clear the local storage, therefore
     * cleaning any stored information like token or claims.
     */
    logout() {
        this.storage.clear();
    }
}
exports.OnixClient = OnixClient;
//# sourceMappingURL=index.js.map